<?php
/**
 * @file
 * Admin settings form and menu callbacks for the reports.
 */

/**
 * Details of a single log entry.
 */
function pagenotfound_reports_details($log) {
  // Very basic table of key/value pairs on the $log object.
  $header = array(
    t('Key'),
    t('Value'),
  );
  $rows = array();
  foreach ($log as $key => $value) {
    $rows[] = array(
      $key,
      $value,
    );
  }

  // Return themed table.
  return theme('table', $header, $rows);
}

/**
 * List of all 404s sorted by most common
 */
function pagenotfound_reports_top_404s() {
  // Start with the stats table.
  $sql = "SELECT MAX(log.timestamp) AS newest, MIN(log.timestamp) AS oldest,
    COUNT(*) AS item_count FROM {pagenotfound_reports_log} log";
  $stats = db_fetch_object(db_query($sql));
  $output = _pagenotfound_reports_summary_stats($stats);

  $header = array(
    array('data' => t('Count'), 'field' => 'item_count', 'sort' => 'desc'),
    array('data' => t('Host'), 'field' => 'parsed_host'),
    array('data' => t('Path'), 'field' => 'parsed_path'),
    array('data' => t('Operations')),
  );

  $sql = "SELECT COUNT(log.parsed_path) AS item_count, log.log_id, log.parsed_path, log.parsed_host
    FROM {pagenotfound_reports_log} log GROUP BY log.parsed_path, log.parsed_host";
  $sql .= tablesort_sql($header);
  $count_sql = "SELECT COUNT(DISTINCT(log.parsed_path)) FROM {pagenotfound_reports_log} log";
  $result = pager_query($sql, 30, 0, $count_sql);

  $rows = array();
  while ($log = db_fetch_object($result)) {
    $rows[] = array(
      $log->item_count,
      $log->parsed_host,
      $log->parsed_path,
      _pagenotfound_reports_add_operations($log),
    );
  }

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No log messages available.'), 'colspan' => 4));
  }

  $output .= theme('table', $header, $rows);
  $output .= theme('pager', NULL, 30, 0);

  return $output;
}

/**
 * List of 404s with referers.
 */
function pagenotfound_reports_referers() {
  // Start with the stats table.
  $sql = "SELECT COUNT(*) AS item_count FROM {pagenotfound_reports_log} log WHERE log.referer_uri != ''";
  $stats = db_fetch_object(db_query($sql));
  // Do NOT call _pagenotfound_reports_summary_stats(), since only 404 errors
  // with referrers are shown in this report.
  $output = theme('table', array(), array(
    array(
      t('Total with referer value'),
      isset($stats->item_count) ? $stats->item_count : 0,
    )
  ));

  $header = array(
    array('data' => t('Count'), 'field' => 'item_count', 'sort' => 'desc'),
    array('data' => t('Host'), 'field' => 'parsed_host'),
    array('data' => t('Path'), 'field' => 'parsed_path'),
    array('data' => t('Referer'), 'field' => 'referer_uri'),
    array('data' => t('Operations')),
  );

  $sql = "SELECT COUNT(log.parsed_path) AS item_count, log.log_id, log.parsed_path, log.parsed_host, log.referer_uri
    FROM {pagenotfound_reports_log} log WHERE log.referer_uri != '' GROUP BY log.parsed_path, log.parsed_host";
  $sql .= tablesort_sql($header);
  $count_sql = "SELECT COUNT(DISTINCT(log.parsed_path)) FROM {pagenotfound_reports_log} log WHERE log.referer_uri != ''";
  $result = pager_query($sql, 30, 0, $count_sql);

  $rows = array();

  $rows = array();
  while ($log = db_fetch_object($result)) {
    $rows[] = array(
      $log->item_count,
      $log->parsed_host,
      $log->parsed_path,
      $log->referer_uri,
      _pagenotfound_reports_add_operations($log),
    );
  }

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No log messages available.'), 'colspan' => 5));
  }

  $output .= theme('table', $header, $rows);
  $output .= theme('pager', NULL, 30, 0);

  return $output;
}

/**
 * List of all 404s in the log
 */
function pagenotfound_reports_all() {
  // Start with the stats table.
  $sql = "SELECT MAX(log.timestamp) AS newest, MIN(log.timestamp) AS oldest,
    COUNT(*) AS item_count FROM {pagenotfound_reports_log} log";
  $stats = db_fetch_object(db_query($sql));
  $output = _pagenotfound_reports_summary_stats($stats);

  $header = array(
    array('data' => t('Host'), 'field' => 'parsed_host'),
    array('data' => t('Path'), 'field' => 'parsed_path'),
    array('data' => t('Timestamp'), 'field' => 'timestamp', 'sort' => 'desc'),
    array('data' => t('Operations')),
  );

  $sql = "SELECT log.log_id, log.parsed_path, log.parsed_host, log.referer_uri, log.timestamp FROM {pagenotfound_reports_log} log";
  $sql .= tablesort_sql($header);
  $count_sql = "SELECT COUNT(log.log_id) FROM {pagenotfound_reports_log} log";
  $result = pager_query($sql, 30, 0, $count_sql);

  $rows = array();

  $rows = array();
  while ($log = db_fetch_object($result)) {
    $rows[] = array(
      $log->parsed_host,
      $log->parsed_path,
      format_date($log->timestamp, 'small'),
      _pagenotfound_reports_add_operations($log),
    );
  }

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No log messages available.'), 'colspan' => 4));
  }

  $output .= theme('table', $header, $rows);
  $output .= theme('pager', NULL, 30, 0);

  return $output;
}

/**
 * Add a set of operation links to a single log entry.
 */
function _pagenotfound_reports_add_operations($log) {
  $operations = array();
  // View details.
  $operations[] = array(
    'title' => t('View details'),
    'href' => 'admin/reports/404/details/' . $log->log_id,
  );
  // Add path_redirect module operations.
  if (!empty($log->parsed_path) && module_exists('path_redirect')) {
    $existing = path_redirect_load_by_source($log->parsed_path);
    if (!empty($existing)) {
      $operations[] = array(
        'title' => t('Edit redirect'),
        'href' => 'admin/build/path-redirect/edit/' . $existing['rid'],
        'query' => drupal_get_destination(),
      );
      $operations[] = array(
        'title' => t('Delete redirect'),
        'href' => 'admin/build/path-redirect/delete/' . $existing['rid'],
        'query' => drupal_get_destination(),
      );
    }
    else {
      $operations[] = array(
        'title' => t('Add redirect'),
        'href' => 'admin/build/path-redirect/add',
        'query' => drupal_get_destination() . '&pagenotfound-path=' . $log->parsed_path,
      );
    }
  }

  return theme('links', $operations);
}

/**
 * Get summary stats to show above the report. Takes the $oldest_item and
 * $item_count variables because different reports include different types or
 * sets of log records, so they need to be figured for each report.
 *
 * @return
 *   Formatted HTML of summary stats to show at the top of the report
 */
function _pagenotfound_reports_summary_stats($stats) {
  $header = array();
  $rows = array();

  $rows[] = array(
    t('Oldest available'),
    isset($stats->oldest) ? format_date($stats->oldest) : t('n/a'),
  );
  $rows[] = array(
    t('Newest available'),
    isset($stats->newest) ? format_date($stats->newest) : t('n/a'),
  );
  $rows[] = array(
    t('Total count'),
    isset($stats->item_count) ? $stats->item_count : 0,
  );

  // Calculate stats.
  if (!empty($stats->oldest) && !empty($stats->item_count)) {
    $rows[] = array(
      t('Errors per minute'),
      round($stats->item_count / ((time() - $stats->oldest) / 60), 2),
    );
  }

  // Return a themed table.
  return theme('table', $header, $rows);
}

/**
 * Page of tips to minimize 404 errors
 */
function pagenotfound_reports_tips() {
  $header = array(
    t('Status'),
    t('Description'),
  );
  
  $rows = array();
  
  if (module_exists('path_redirect')) {
    $row = array(
      'data' => array(
        t('Done'),
        t('Path Redirect module is installed, so that redirects can be created from the 404 reports.'),
      ),
      'class' => 'ok',
    );
  }
  else {
    $row = array(
      'data' => array(
        t('Todo'),
        t('Install the !path_redirect module so that redirects can be quickly and easily created for any item in the 404 reports.', array('!path_redirect' => l('Path Redirect', 'http://drupal.org/project/path_redirect'))),
      ),
      'class' => 'warning',
    );
  }
  $rows[] = $row;
  
  if (module_exists('advagg')) {
    $row = array(
      'data' => array(
        t('Done'),
        t('Advanced CSS/JS Aggregation module is installed, which will reduce 404 errors related to stale CSS and JS cache files.'),
      ),
      'class' => 'ok',
    );    
  }
  else {
    $row = array(
      'data' => array(
        t('Todo'),
        t('Install the !advagg module to reduce 404 errors related to stale CSS and JS cache files.', array('!advagg' => l('Advanced CSS/JS Aggregation', 'http://drupal.org/project/advagg'))),
      ),
      'class' => 'warning',
    );
  }
  $rows[] = $row;
  
  return theme('table', $header, $rows);
}

/**
 * Page Not Found Reports module settings form.
 */
function pagenotfound_reports_admin_settings() {
  $form['pagenotfound_reports_log_row_limit'] = array(
    '#type' => 'select',
    '#title' => t('Discard log entries above the following row limit'),
    '#default_value' => variable_get('pagenotfound_reports_log_row_limit', 1000),
    '#options' => drupal_map_assoc(array(100, 1000, 10000, 100000, 1000000)),
    '#description' => t('The maximum number of rows to keep in the log. Older entries will be automatically discarded.'),
  );
  return system_settings_form($form);
}
